Attribute VB_Name = "modUsrFunctions"
'User Functions Module
'
'This module contains the functions for dealing with functions, as well as the
'function container, which is 'dProcs'.
'In this code, I use the term 'proc', which is short for procedure.  Why I did this
'instead of functions I haven't a clue, as it was always intended to be called functions

Option Explicit

Public Type typeProc
    Name As String
    Parameters As String
    Code As String
    Location As Long
    
    isArray As Boolean
End Type

Public dProcs() As typeProc, ProcCount As Integer

Public Sub ArrangeProcs(ByVal Script As String)
    On Error GoTo GeneralError
    
    Dim TempProc As typeProc
    Dim tempInt As Integer
    Dim TempArray() As String
    Dim i As Long, z As Integer
    Dim numElements As Integer
    
    Dim FoundProc As Boolean
    
    For i = 1 To Len(Script)
    
        'make sure it's not a space, return, wierd character, etc...
        tempInt = Asc(Mid(Script, i, 1))
        If tempInt < 36 Then GoTo skip
        If tempInt > 192 Then GoTo skip
    
        'check if it's a quote
        If Mid(Script, i, 1) = Chr(34) Then
            If StringExist(i, Script, Chr(34)) Then
                i = InStr(i + 1, Script, Chr(34))
            Else
                FlagError "Syntax Error", "Expected end-quote", i: Exit Sub
            End If
        End If
        
        If LCase(Mid(Script, i, 9)) = "function " Then
            i = i + 9
            
            'get function name
            tempInt = InStr(i, Script, "(")
            TempProc.Name = Trim(Mid(Script, i, tempInt - i))
            TempProc.isArray = False
            
            If InStr(1, TempProc.Name, "[") <> 0 Then
                parseArrayVar TempProc.Name, numElements
                If numElements <> -1 Then _
                    FlagError "Syntax Error", "Function '" & TempProc.Name & "' cannot return a fixed-size array.", i
                TempProc.isArray = True
            End If
            If Left(TempProc.Name, 4) = "dpp." Then _
                FlagError "Syntax Error", "Cannot declare function '" & TempProc.Name & "': " & vbCrLf & "The name contains reserved keywords.", i
            i = tempInt + 1
            
            'get function parameters
            tempInt = InStr(i, Script, ")")
            TempProc.Parameters = Trim(Mid(Script, i, tempInt - i))
            i = tempInt + 1
            
            If TempProc.Name = "main" And TempProc.Parameters <> "" Then GoTo MainError
            
            'get function code
            i = InStr(i, Script, "{") + 1
            TempProc.Location = i
            tempInt = InStr(i, Script, "}")
            TempProc.Code = Mid(Script, i, tempInt - i)
            i = tempInt
            
            If Trim(TempProc.Name) = "" Then GoTo skip

            If FindProc(TempProc.Name) Then _
                FlagError "Link Error", "Function declaration error!  Ambiguous name detected." & vbCrLf & vbCrLf & "Function: " & TempProc.Name, i: Exit Sub
            
            'reinitialize the array
            ProcCount = ProcCount + 1
            ReDim Preserve dProcs(ProcCount)
            
            
            dProcs(ProcCount) = TempProc
            
            FoundProc = True
            
        'for global variables
        ElseIf LCase(Mid(Script, i, 7)) = "newvar " Then
            i = i + 7
           
            If FoundProc Then FlagError "Link Error", "Invalid variable declaration placement.  Global variables must be placed at top of code.", i: Exit Sub
            
            TempArray = dSplit(GetValue(i, ";", False, Script), ",")
            For z = LBound(TempArray) To UBound(TempArray)
                tempInt = InStr(1, TempArray(z), "=")
                If tempInt = 0 Then
                    AddVar Trim(TempArray(z)), "", "dpp.global", True
                Else
                    AddVar Trim(Mid(TempArray(z), 1, tempInt - 1)), Trim(Mid(TempArray(z), tempInt + 1)), "dpp.global", True
                End If
            Next z
        ElseIf FoundProc Then
            FlagError "Syntax Error", "Invalid statement.  Only comments may appear between functions.", i: Exit Sub
            
        End If
        
skip:
    Next 'i
    
    Exit Sub
    
GeneralError:
    If TempProc.Name = "" Then TempProc.Name = "Unknown"
    FlagError "Runtime Error", "General error during function intialization: " & err.Description & vbCrLf & vbCrLf & "Function Definition: " & TempProc.Name, i
    Exit Sub
    
ProcError:
    If TempProc.Name = "" Then TempProc.Name = "Unknown"
    FlagError "Syntax Error", "Function declaration error!" & vbCrLf & vbCrLf & "Function Definition: " & TempProc.Name, i
    Exit Sub
    
MainError:
    FlagError "Syntax Error", "Function declaration error!  'Main' cannot hold parameters!" & vbCrLf & vbCrLf & "Function Definition: main", i
    Exit Sub
End Sub

Function GetProcCode(ProcName As String) As String
    'gets the function's code
    Dim i As Integer
    
    For i = 0 To ProcCount
        If LCase(dProcs(i).Name) = LCase(ProcName) Then
            GetProcCode = dProcs(i).Code
            Exit Function
        End If
    Next 'i
    
    FlagError "Syntax Error", "Function undefined: " & ProcName, iLocation: Exit Function
End Function

Function GetProcLoc(ProcName As String) As String
    'gets the function's location
    Dim i As Integer
    
    For i = 1 To ProcCount
        If LCase(dProcs(i).Name) = LCase(ProcName) Then
            GetProcLoc = dProcs(i).Location
            Exit Function
        End If
    Next 'i
    
    FlagError "Syntax Error", "Function undefined: " & ProcName, iLocation: Exit Function
End Function

Function GetProcParams(ProcName As String) As String
    'gets the function's parameters
    Dim i As Integer
    
    For i = 1 To ProcCount
        If LCase(dProcs(i).Name) = LCase(ProcName) Then
            GetProcParams = dProcs(i).Parameters
            Exit Function
        End If
    Next 'i
    
    FlagError "Syntax Error", "Function undefined: " & ProcName, iLocation: Exit Function
End Function

Function FindProc(ProcName As String) As Boolean
    'determines if a function exists
    Dim i As Integer
    
    For i = 1 To ProcCount
        If LCase(dProcs(i).Name) = LCase(ProcName) Then
            FindProc = True
            Exit Function
        End If
    Next 'i
End Function

Function GetProcIndex(ProcName As String, ShowError As Boolean) As Integer
    'get's a function's index
    Dim i As Integer
    
    For i = 0 To ProcCount
        If LCase(dProcs(i).Name) = LCase(ProcName) Then
            GetProcIndex = i
            Exit Function
        End If
    Next 'i
    
    If ShowError Then _
        FlagError "Syntax Error", "Function undefined: " & ProcName, iLocation: Exit Function
        
    GetProcIndex = -1
End Function

Function ExecuteFunction(ByVal ProcName As String, Vars() As typeVar)
'executes a function
Dim OldCode As String, OldProc As String
Dim NewCode As String
Dim Args As Variant, i As Integer
Dim uArgs As Integer
            
If Not FindProc(ProcName) Then
    FlagError "Syntax Error", "Function undefined (EF): " & ProcName, iLocation: Exit Function
    Exit Function
End If

NewCode = GetProcCode(ProcName)  'get the function's code from code array
                    
OldCode = dScript           'save the old code
OldProc = CurrentProc       'save the old proc
CurrentProc = ProcName     'set the current proc

Args = Split(GetProcParams(ProcName), ",") 'split declaration arguments

uArgs = UBound(Vars)
If Trim(Vars(0).Name) = "" Then uArgs = -1

If UBound(Args) <> uArgs Then
    FlagError "Syntax Error", "Wrong number of arguments for " & ProcName & "().", iLocation: Exit Function
    Exit Function
End If

If uArgs > -1 Then
    For i = LBound(Vars) To UBound(Vars)
        AddVar Trim(Vars(i).Name), Vars(i).Value, CurrentProc, False
    Next i
End If

'parse all new code, based on this function
frmRun.LinkCode NewCode, False

ClearVars ProcName      'clear all vars for this function
CurrentProc = OldProc   'reset to old proc
dScript = OldCode       'reset to old code
End Function

